@@ -121,11 +121,84 @@ MGLMsparsereg <- function(formula, data, dist, lambda, penalty, weight,
 MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx, 
 			maxiters=150, ridgedelta, epsilon=1e-5, regBeta=FALSE, overdisp){
 
+  ##---------------- Begin of the MGLM.loss function ----------------##
+  MGLM.loss <- function(beta, dist, regBeta=FALSE, Beta){
+    if(dist=="MN"){
+      P <- matrix(NA, N, d)
+      P[, d] <- rep(1, N)
+      P[, 1:(d-1)] <- exp(X%*%beta)
+      P <- P/rowSums(P)
+      loss <- -sum(weight*dmn(Y, P))
+      kr1 <- Y[,-d]-P[, 1:(d-1)]*m
+      if(d>2){
+        lossD1 <- -colSums(kr(kr1, X, weight))
+      }else if(d==2){
+        lossD1 <- -colSums( kr1*weight*X)
+      }
+      lossD1 <- matrix(lossD1, p, (d-1))
+    }else if(dist=="DM"){
+      alpha <- exp(X%*%beta)
+      loss <- -sum(weight*ddirm(Y, alpha))
+      tmpvector <- digamma(rowSums(alpha) + m) - digamma(rowSums(alpha))
+      tmpmatrix <- digamma(alpha+ Y) - digamma(alpha)
+      dalpha <- tmpmatrix - tmpvector
+      lossD1 <- - colSums(kr(alpha*dalpha, X, weight))
+      lossD1 <- matrix(lossD1, p, d)
+    }else if(dist=="GDM"){
+      Ys <- t( apply(apply( apply(Y,1,rev),2,cumsum),2,rev) )
+      alpha <- exp(X%*%beta)
+      A <- alpha[, 1:(d-1)]
+      B <- alpha[, d:(2*(d-1))]
+      loss <- -sum(weight*dgdirm(Y, A,B))
+      dalpha1 <- digamma(A+ Y[,-d])-digamma(A)-digamma(A+B+Ys[,-d])+digamma(A+B)
+      dalpha2 <- digamma(B+Ys[,-1])-digamma(B)-digamma(A+B+Ys[,-d])+digamma(A+B)
+      dalpha <- cbind(dalpha1, dalpha2)
+      lossD1 <- -colSums(kr(alpha*dalpha, X, weight))
+      lossD1 <- matrix(lossD1, p, 2*(d-1))
+    }else if(dist=="NegMN"){
+      if(regBeta){
+        P <- matrix(NA, N, (d+1))
+        alpha <- exp(X%*%beta)
+        Beta <- alpha[,(d+1)]
+        alpha_rowsums <- rowSums(alpha[, 1:d]) + 1
+        P[,(d+1)] <- 1/alpha_rowsums
+        P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
+        loss <- - sum(weight* dneg(Y, alpha[, 1:d], Beta) )
+        deta <- matrix(0, nrow(Y), d+1)
+        deta[, 1:d] <- Y-alpha[,1:d]*Beta-P[,1:d]*(m-(alpha_rowsums-1)*Beta)
+        deta[, (d+1)] <- Beta*(digamma(Beta+m)-digamma(Beta)+log(P[, (d+1)]))
+        lossD1 <- -colSums(kr(deta, X, weight))
+        lossD1 <- matrix(lossD1, p, (d+1))
+      }else{
+        P <- matrix(NA, N, (d+1))
+        alpha <- exp(X%*%beta)
+        alpha_rowsums <- rowSums(alpha) + 1
+        P[,(d+1)] <- 1/alpha_rowsums
+        P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
+        loss <- - sum(weight* dneg(Y, alpha, rep(Beta, N)) )
+        deta <- Y - (Beta + m)*P[,1:d]
+        
+        lossD1 <- - colSums(kr(deta, X, weight))
+        lossD1 <- matrix(lossD1, p, d)
+      }
+    }
+    
+    return(list(loss, lossD1))
+  }
+  ##---------------- End of the MGLM.loss function ----------------##
+  
+  
 	d <- ncol(Y)
 	m <- rowSums(Y)
 	p <- ncol(X)
 	N <- nrow(Y)
 
+	if(dist=="GDM"){
+		colOrder <- order(colSums(Y), decreasing=TRUE)
+		Y <- Y[, colOrder]
+		outOrder <- order(colOrder[-d])
+		#Ys <- t( apply(apply( apply(Y,1,rev),2,cumsum),2,rev) )
+	}
 	##----------------------------------------##
 	## Get prepared
 	beta_old <- init
@@ -138,16 +211,19 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 	oriRidge <- ridgedelta
 	##----------------------------------------##
 	## Main loop
+#  browser()
 	while( (niter<3) || (!stop) ){
-#	  browser()  
+    if(niter==20) break
+    ridgedelta <- oriRidge
 		niter <- niter+1		
 		beta_old <- B
 		obj1 <- objval
 		##----------------------------------------##
 		## Current search point
 		if(niter<=2) S <- init		
-		loss <- MGLM.loss(Y, X, S, dist, weight, regBeta, overdisp)
-		loss.S <- loss[[1]]
+#		loss <- MGLM.loss(Y, X, S, dist, weight, regBeta, overdisp)
+    loss <- MGLM.loss(S, dist, regBeta, overdisp)
+    loss.S <- loss[[1]]
 		loss.D1S <- loss[[2]]
 		##----------------------------------------##
 		## Line search	
@@ -155,18 +231,21 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 			A <- S - ridgedelta*loss.D1S
 			B <- matrix(NA, nrow(B), ncol(B))
 			B[!penidx, ] <- A[!penidx,]
-      if(d>2) Apen <- A[penidx, ] 
-      else if(d==2) Apen <- matrix(A[penidx,], , 1) 
+      if(d>2){
+        Apen <- A[penidx, ] 
+      }else if(d==2){
+        Apen <- matrix(A[penidx,], , 1) 
+      }
 			pen <- matrix_threshold(X=Apen, lambda=ridgedelta*lambda, 
 					penalty=penalty)
 			B[penidx, ] <- pen[[1]]
 			penval <- pen[[2]]
 			##----------------------------------------##
 			## Find maxlambda
-			if( all(abs(B)==0) ){
+			if( all(B[penidx, ]==0) ){
 				if(penalty=="sweep"){
 					maxlambda <- max(abs(Apen))/ridgedelta
-				}else if(penalty=="group_row"){
+				}else if(penalty=="group_row" ||penalty=="group" ){
 					maxlambda <- max(sqrt(rowSums(Apen^2)))/ridgedelta
 				}else if(penalty=="nuclear"){
 					sing.vec <- svd(Apen)$d
@@ -182,7 +261,7 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 			
 			##----------------------------------------##
 			## Objective value 
-			objloss <- MGLM.loss(Y,X, B, dist, weight, regBeta, overdisp)
+			objloss <- MGLM.loss(B, dist, regBeta, overdisp)			
 			loss.S2 <- objloss[[1]] 
 			loss.D1S2 <- objloss[[2]] 
 			objval <- loss.S2 + penval
@@ -192,22 +271,28 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 			BminusS <- B-S
 			surval <- loss.S + sum(loss.D1S*BminusS)+
 					norm(BminusS, type="F")^2/2/ridgedelta + penval	
-				
+
 			##----------------------------------------##
 			## Line search stopping rule
-			if(!is.na(objval)&&!is.na(surval))
-				if(objval <= surval) break else ridgedelta <- ridgedelta/2
-
+			if(!is.na(objval)&&!is.na(surval)){
+				if(objval <= surval){
+          break
+        }else{
+          ridgedelta <- ridgedelta/2
+        }
+			}else if(is.na(objval)){
+			  ridgedelta <- ridgedelta/2
+			}
+      
 		}
 		##----------------------------------------##
 		## End of line search.
-
 		alpha_old <- alpha
 		alpha <- (1+ sqrt(4 + alpha_old^2))/2	
 		
 		##----------------------------------------##
 		## force descent
-		if(objval<=obj1){ ## descent
+		if(!is.na(objval) & objval<=obj1){ ## descent
 			stop <- abs(obj1-objval)<epsilon*(abs(obj1)+1)	
 			S <- B + (alpha_old-1)/alpha*(B-beta_old)
 		}else{
@@ -221,7 +306,6 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 				stop <- TRUE
 			}
 		} 
-
 		##----------------------------------------##
 		## Maxiter
 		if(niter>=maxiters) stop <- TRUE
@@ -233,11 +317,10 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 	## Calculate Dof
   if(d>2) Bpen <- B[penidx, ]
   else if(d==2) Bpen <- matrix(B[penidx, ], , 1)
-  
 	if(penalty=="sweep"){
 		Dof <- sum(penidx==FALSE)*ncol(B) + sum( Bpen!= 0)
 	}else if(penalty=="group_row"){
-		Dof <- sum(!penidx)*ncol(B) + sum(rowSums(Bpen^2)!=0) + 
+		Dof <- sum(!penidx)*ncol(B) + sum(rowSums(Bpen^2)>0) + 
 				sum((ncol(B)-1)*rowSums(Bpen^2)/rowSums(Apen^2))
 	}else if(penalty=="nuclear"){
 		Aspectrum <- svd(A[penidx,])$d
@@ -254,15 +337,18 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 			Dof <- Dof+sum(!penidx)*d
 		}
 	}
-	
 	if(dist=="GDM") Dof <- Dof/2 else Dof <- Dof
 	
 	##----------------------------------------##
 	## Output other statistics
-	logL <- - MGLM.loss(Y, X, B, dist, weight, regBeta, overdisp)[[1]]
-	AIC <- -2*logL + 2*Dof
+#	logL <- - MGLM.loss(Y, X, B, dist, weight, regBeta, overdisp)[[1]]
+  logL <- - MGLM.loss(B, dist, regBeta, overdisp)[[1]]
+  AIC <- -2*logL + 2*Dof
 	BIC <- -2*logL + log(N)*Dof
 		
+	if(dist=="GDM"){
+		B <- B[, c(outOrder, outOrder+d-1)]
+	}	
 	return(list(coefficients=B, logL=logL, AIC=AIC, BIC=BIC, Dof=Dof, 
 		iter=niter,	maxlambda=maxlambda))
 }
@@ -270,89 +356,81 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
 
 
 
-##============================================================================##
-## MGLM logss
-##============================================================================##
-
-MGLM.loss <- function(Y, X, beta, dist, weight, regBeta=FALSE, Beta){
-	
-	N <- nrow(Y)
-	d <- ncol(Y)
-	p <- ncol(X)
-	m <- rowSums(Y)
-	if(missing(weight)) weight <- rep(1, N)
-	
-	if(dist=="MN"){
-		P <- matrix(NA, N, d)
-		P[, d] <- rep(1, N)
-		P[, 1:(d-1)] <- exp(X%*%beta)
-		P <- P/rowSums(P)
-		loss <- -sum(weight*dmn(Y, P))
-		kr1 <- Y[,1:(d-1)]-P[, 1:(d-1)]*m
-    if(d>2){
-#		  lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B, w) 
-#							return(w[i]*A[i,]%x%B[i,]), kr1, X, weight))
-      lossD1 <- -colSums(kr(kr1, X, weight))
-    }else if(d==2){
-      lossD1 <- -colSums( kr1*weight*X)
-    }
-		lossD1 <- matrix(lossD1, p, (d-1))
-	}else if(dist=="DM"){
-		alpha <- exp(X%*%beta)
-		loss <- -sum(weight*ddirm(Y, alpha))
-		tmpvector <- digamma(rowSums(alpha) + m) - digamma(rowSums(alpha))
-		tmpmatrix <- digamma(alpha+ Y) - digamma(alpha)
-		dalpha <- tmpmatrix - tmpvector
-#		lossD1 <- - rowSums(sapply(1:nrow(Y), function(i, A, B,w) 
-#							return(w[i]*A[i,]%x%B[i,]), alpha*dalpha, X, weight) )
-    lossD1 <- - colSums(kr(alpha*dalpha, X, weight))
-		lossD1 <- matrix(lossD1, p, d)
-	}else if(dist=="GDM"){
-		Ys <- t( apply(apply( apply(Y,1,rev),2,cumsum),2,rev) )
-		alpha <- exp(X%*%beta)
-		A <- alpha[, 1:(d-1)]
-		B <- alpha[, d:(2*(d-1))]
-		loss <- -sum(weight*dgdirm(Y, A,B))
-		dalpha1 <- digamma(A+ Y[,-d])-digamma(A)-digamma(A+B+Ys[,-d])+digamma(A+B)
-		dalpha2 <- digamma(B+Ys[,-1])-digamma(B)-digamma(A+B+Ys[,-d])+digamma(A+B)
-		dalpha <- cbind(dalpha1, dalpha2)
-#		lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B,w) 
-#							return(w[i]*A[i,]%x%B[i,]), alpha*dalpha, X, weight) )
-    lossD1 <- -colSums(kr(alpha*dalpha, X, weight))
-		lossD1 <- matrix(lossD1, p, 2*(d-1))	
-	}else if(dist=="NegMN"){
-		if(regBeta){
-			P <- matrix(NA, N, (d+1))
-			alpha <- exp(X%*%beta)
-			Beta <- alpha[,(d+1)]
-			alpha_rowsums <- rowSums(alpha[, 1:d]) + 1
-			P[,(d+1)] <- 1/alpha_rowsums
-			P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
-			loss <- - sum(weight* dneg(Y, alpha[, 1:d], Beta) )
-			deta <- matrix(0, nrow(Y), d+1)
-			deta[, 1:d] <- Y-alpha[,1:d]*Beta-P[,1:d]*(m-(alpha_rowsums-1)*Beta)
-			deta[, (d+1)] <- Beta*(digamma(Beta+m)-digamma(Beta)+log(P[, (d+1)]))
-#			lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B,w) 
-#								return(w[i]*A[i,]%x%B[i,]), deta, X, weight) )
-      lossD1 <- -colSums(kr(deta, X, weight))
-			lossD1 <- matrix(lossD1, p, (d+1))
-		}else{
-			P <- matrix(NA, N, (d+1))
-			alpha <- exp(X%*%beta)
-			alpha_rowsums <- rowSums(alpha) + 1
-			P[,(d+1)] <- 1/alpha_rowsums
-			P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
-			loss <- - sum(weight* dneg(Y, alpha, rep(Beta, N)) )
-			deta <- Y - (Beta + m)*P[,1:d]
-#			lossD1 <-  -rowSums(sapply(1:N, function(i, A, B,w) 
-#								return(w[i]*A[i,]%x%B[i,]), deta, X, weight) )
-      lossD1 <- - colSums(kr(deta, X, weight))
-			lossD1 <- matrix(lossD1, p, d)
-		}
-	}
-	
-	return(list(loss, lossD1))
-}
+# ##============================================================================##
+# ## MGLM logss
+# ##============================================================================##
+# 
+# MGLM.loss <- function(beta, dist, regBeta=FALSE, Beta){
+# #MGLM.loss <- function(Y, X, beta, dist, weight, regBeta=FALSE, Beta){
+# 	
+# # 	N <- nrow(Y)
+# # 	d <- ncol(Y)
+# # 	p <- ncol(X)
+# # 	m <- rowSums(Y)
+# # 	if(missing(weight)) weight <- rep(1, N)
+# 	
+# 	if(dist=="MN"){
+# 		P <- matrix(NA, N, d)
+# 		P[, d] <- rep(1, N)
+# 		P[, 1:(d-1)] <- exp(X%*%beta)
+# 		P <- P/rowSums(P)
+# 		loss <- -sum(weight*dmn(Y, P))
+# 		kr1 <- Y[,1:(d-1)]-P[, 1:(d-1)]*m
+#     if(d>2){
+#       lossD1 <- -colSums(kr(kr1, X, weight))
+#     }else if(d==2){
+#       lossD1 <- -colSums( kr1*weight*X)
+#     }
+# 		lossD1 <- matrix(lossD1, p, (d-1))
+# 	}else if(dist=="DM"){
+# 		alpha <- exp(X%*%beta)
+# 		loss <- -sum(weight*ddirm(Y, alpha))
+# 		tmpvector <- digamma(rowSums(alpha) + m) - digamma(rowSums(alpha))
+# 		tmpmatrix <- digamma(alpha+ Y) - digamma(alpha)
+# 		dalpha <- tmpmatrix - tmpvector
+#     lossD1 <- - colSums(kr(alpha*dalpha, X, weight))
+# 		lossD1 <- matrix(lossD1, p, d)
+# 	}else if(dist=="GDM"){
+# 		Ys <- t( apply(apply( apply(Y,1,rev),2,cumsum),2,rev) )
+# 		alpha <- exp(X%*%beta)
+# 		A <- alpha[, 1:(d-1)]
+# 		B <- alpha[, d:(2*(d-1))]
+# 		loss <- -sum(weight*dgdirm(Y, A,B))
+# 		dalpha1 <- digamma(A+ Y[,-d])-digamma(A)-digamma(A+B+Ys[,-d])+digamma(A+B)
+# 		dalpha2 <- digamma(B+Ys[,-1])-digamma(B)-digamma(A+B+Ys[,-d])+digamma(A+B)
+# 		dalpha <- cbind(dalpha1, dalpha2)
+#     lossD1 <- -colSums(kr(alpha*dalpha, X, weight))
+# 		lossD1 <- matrix(lossD1, p, 2*(d-1))	
+# 	}else if(dist=="NegMN"){
+# 		if(regBeta){
+# 			P <- matrix(NA, N, (d+1))
+# 			alpha <- exp(X%*%beta)
+# 			Beta <- alpha[,(d+1)]
+# 			alpha_rowsums <- rowSums(alpha[, 1:d]) + 1
+# 			P[,(d+1)] <- 1/alpha_rowsums
+# 			P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
+# 			loss <- - sum(weight* dneg(Y, alpha[, 1:d], Beta) )
+# 			deta <- matrix(0, nrow(Y), d+1)
+# 			deta[, 1:d] <- Y-alpha[,1:d]*Beta-P[,1:d]*(m-(alpha_rowsums-1)*Beta)
+# 			deta[, (d+1)] <- Beta*(digamma(Beta+m)-digamma(Beta)+log(P[, (d+1)]))
+#       lossD1 <- -colSums(kr(deta, X, weight))
+# 			lossD1 <- matrix(lossD1, p, (d+1))
+# 		}else{
+# 			P <- matrix(NA, N, (d+1))
+# 			alpha <- exp(X%*%beta)
+# 			alpha_rowsums <- rowSums(alpha) + 1
+# 			P[,(d+1)] <- 1/alpha_rowsums
+# 			P[, 1:d] <- alpha[, 1:d]*P[, (d+1)]
+# 			loss <- - sum(weight* dneg(Y, alpha, rep(Beta, N)) )
+# 			deta <- Y - (Beta + m)*P[,1:d]
+# 
+#       lossD1 <- - colSums(kr(deta, X, weight))
+# 			lossD1 <- matrix(lossD1, p, d)
+# 		}
+# 	}
+# 	
+# 	return(list(loss, lossD1))
+# }
 
 
 ##============================================================================##
@@ -363,11 +441,11 @@ matrix_threshold <- function(X, lambda, penalty){
 	
 	N <- nrow(X)
 	d <- ncol(X)
-#	B <- matrix(0, N, d)
+	B <- matrix(0, N, d)
 	if(penalty=="sweep"){
 		B <- lsq_thresholding(X, lambda)
 		penalty_value <- lambda*sum(abs(B))
-	}else if(penalty=="group_row"){
+	}else if(penalty=="group_row" || penalty=="group"){
 		row_12norm <- sqrt(rowSums(X^2))
 		vec  <- 1-lambda/row_12norm
 		vec[vec<0] <- 0
